@@ -106,7 +106,7 @@ char *garmin_fs_xstrdup(const char *src, size_t size);
 
 /* for GPX */
 void garmin_fs_xml_convert(const int base_tag, int tag, const char *cdatastr, waypoint *waypt);
-void garmin_fs_xml_fprint(FILE *ofd, const waypoint *waypt);
+void garmin_fs_xml_fprint(gbfile *ofd, const waypoint *waypt);
 
 /* common garmin_fs utilities */
 
