@@ -1,8 +1,7 @@
 /*
 
     Common GPSBabel file I/O API
-
-    Copyright (C) 2006 Olaf Klein 
+    Copyright (C) 2006 Olaf Klein, o.b.klein@gpsbabel.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -40,7 +39,7 @@
 #endif
 
 #define MYNAME "gbfile"
-#define NO_ZLIB MYNAME ": No zlib support\n."
+#define NO_ZLIB MYNAME ": No zlib support.\n"
 
 /* About the ZLIB_INHIBITED stuff:
  * 
@@ -53,6 +52,10 @@
 
 /* GPSBabel 'file' standard calls */
 
+/*
+ * gbfopen: (as xfopen) plus the name of the calling GPSBabel module (MYNAME)
+ */
+
 gbfile *
 gbfopen(const char *filename, const char *mode, const char *module)
 {
@@ -67,6 +70,7 @@ gbfopen(const char *filename, const char *mode, const char *module)
 	file->line = xstrdup("");
 	file->mode = 'r'; // default
 	file->binary = (strchr(mode, 'b') != NULL);
+	file->back = -1;
 	
 	for (m = mode; *m; m++) {
 		switch(tolower(*m)) {
@@ -85,8 +89,12 @@ gbfopen(const char *filename, const char *mode, const char *module)
 	/* Do we have a '.gz' extension in the filename ? */
 	len = strlen(file->name);
 	if ((len > 3) && (case_ignore_strcmp(&file->name[len-3], ".gz") == 0)) {
+#if !ZLIB_INHIBITED
 		/* force gzipped files on output */
 		file->gzapi = 1;
+#else
+		fatal(NO_ZLIB);
+#endif
 	}
 	
 	if (file->gzapi) {
@@ -151,6 +159,10 @@ gbfopen_be(const char *filename, const char *mode, const char *module)
 	return result;
 }
 
+/*
+ * gbfclose: (as fclose)
+ */
+ 
 void
 gbfclose(gbfile *file)
 {
@@ -173,6 +185,10 @@ gbfclose(gbfile *file)
 	xfree(file);
 }
 
+/*
+ * gbfgetc: (as fgetc)
+ */
+ 
 int 
 gbfgetc(gbfile *file)
 {
@@ -187,6 +203,10 @@ gbfgetc(gbfile *file)
 	}
 }
 
+/*
+ * gbfgets: (as fgets)
+ */
+ 
 char * 
 gbfgets(char *buf, int len, gbfile *file)
 {
@@ -212,7 +232,10 @@ gbfgets(char *buf, int len, gbfile *file)
 	return (*result != '\0') ? result : NULL;
 }
 
-
+/*
+ * gbfread: (as fread)
+ */
+ 
 gbsize_t
 gbfread(void *buf, const gbsize_t size, const gbsize_t members, gbfile *file)
 {
@@ -220,14 +243,29 @@ gbfread(void *buf, const gbsize_t size, const gbsize_t members, gbfile *file)
 	
 	if (file->gzapi) {
 #if !ZLIB_INHIBITED
-		int result;
-		result = gzread(file->handle.gz, buf, size * members) / size;
+		int result = 0;
+		char *target = buf;
+		int count = size * members;
+		
+		if (file->back != -1) {
+			*target++ = file->back;
+			count--;
+			result++;
+			file->back = -1;
+		}
+		result += gzread(file->handle.gz, target, count);
+		result /= size;
 
 		if ((result < 0) || ((gbsize_t)result < members)) {
 			int errnum;
 			const char *errtxt;
 			
 			errtxt = gzerror(file->handle.gz, &errnum);
+			
+			/* Workaround for zlib bug: buffer error on empty files */
+			if ((errnum == Z_BUF_ERROR) && (gztell(file->handle.gz) == 0)) {
+				return (gbsize_t) 0;
+			}
 			if ((errnum != Z_STREAM_END) && (errnum != 0))
 				fatal("%s: zlib returned error %d ('%s')!\n",
 					file->module, errnum, errtxt);
@@ -250,6 +288,10 @@ gbfread(void *buf, const gbsize_t size, const gbsize_t members, gbfile *file)
 	}
 }
 
+/*
+ * gbfprintf: (as fprintf)
+ */
+ 
 int 
 gbfprintf(gbfile *file, const char *format, ...)
 {
@@ -287,6 +329,10 @@ gbfprintf(gbfile *file, const char *format, ...)
 	return gbfwrite(file->buff, 1, len, file);
 }
 
+/*
+ * gbfputc: (as fputc)
+ */
+ 
 int 
 gbfputc(int c, gbfile *file)
 {
@@ -297,12 +343,20 @@ gbfputc(int c, gbfile *file)
 	return c;
 }
 
+/*
+ * gbfputs: (as fputs)
+ */
+
 int 
 gbfputs(const char *s, gbfile *file)
 {
 	return gbfwrite(s, 1, strlen(s), file);
 }
 
+/*
+ * gbfwrite: (as fwrite)
+ */
+
 int 
 gbfwrite(const void *buf, const gbsize_t size, const gbsize_t members, gbfile *file)
 {
@@ -332,6 +386,10 @@ gbfwrite(const void *buf, const gbsize_t size, const gbsize_t members, gbfile *f
 	return result;
 }
 
+/*
+ * gbfflush: (as fflush)
+ */
+ 
 int
 gbfflush(gbfile *file)
 {
@@ -348,12 +406,18 @@ gbfflush(gbfile *file)
 	}
 }
 
+/*
+ * gbfclearerr: (as clearerr)
+ */
+
 void
 gbfclearerr(gbfile *file)
 {
 	if (file->gzapi) {
 #if !ZLIB_INHIBITED
 		gzclearerr(file->handle.gz);
+#else
+		fatal(NO_ZLIB);
 #endif
 	}
 	else {
@@ -361,6 +425,10 @@ gbfclearerr(gbfile *file)
 	}
 }
 
+/*
+ * gbferror: (as ferror)
+ */
+ 
 int
 gbferror(gbfile *file)
 {
@@ -380,6 +448,10 @@ gbferror(gbfile *file)
 	return errnum;
 }
 
+/*
+ * gbfrewind: (as frewind)
+ */
+ 
 void
 gbfrewind(gbfile *file)
 {
@@ -387,6 +459,10 @@ gbfrewind(gbfile *file)
 	gbfclearerr(file);
 }
 
+/*
+ * gbfseek: (as fseek)
+ */
+ 
 int
 gbfseek(gbfile *file, gbint32 offset, int whence)
 {
@@ -397,12 +473,17 @@ gbfseek(gbfile *file, gbint32 offset, int whence)
 		assert(whence != SEEK_END);
 
 #if !ZLIB_INHIBITED
+		if ((whence == SEEK_CUR) && (file->back != -1)) offset--;
 		result = gzseek(file->handle.gz, offset, whence);
+		file->back = -1;
 #else
 		result = 1;
 #endif
-		is_fatal(result < 0,
-			"%s: online compression not yet supported for this format!", file->module);
+		if (result < 0) {
+			if (strcmp(file->name, "-") == 0)
+				fatal("%s: This format cannot be used in piped commands!\n", file->module);
+			fatal("%s: online compression not yet supported for this format!", file->module);
+		}
 		return 0;
 		
 	}
@@ -411,12 +492,21 @@ gbfseek(gbfile *file, gbint32 offset, int whence)
 	}
 }
 
+/*
+ * gbftell: (as ftell)
+ */
+
 gbsize_t 
 gbftell(gbfile *file)
 {
 	if (file->gzapi) {
 #if !ZLIB_INHIBITED
-		return gztell(file->handle.gz);
+		gbsize_t result = gztell(file->handle.gz);
+		if (file->back != -1) {
+			file->back = -1;
+			result--;
+		}
+		return result;
 #else
 		fatal(NO_ZLIB);
 		return -1;
@@ -427,12 +517,37 @@ gbftell(gbfile *file)
 	}
 }
 
+/*
+ * gbfeof: (as feof)
+ */
+
 int 
 gbfeof(gbfile *file)
 {
 	if (file->gzapi) {
 #if !ZLIB_INHIBITED
-		return gzeof(file->handle.gz);
+		int res;
+		
+		if (file->back != -1) return 0;
+
+		res  = gzeof(file->handle.gz);
+		if (!res) {
+			signed char test;
+			int len = gzread(file->handle.gz, &test, 1);
+			if (len == 1) {
+				/* No EOF, put the single byte back into stream */
+				file->back = test;
+			}
+			else {
+				/* we are at the end of the file */
+				if (global_opts.debug_level > 0) {
+					/* now gzeof() should return 1 */
+					is_fatal(!gzeof(file->handle.gz), "zlib gzeof error!\n");
+				}
+				res = 1;
+			}
+		}
+		return res;
 #else
 		fatal(NO_ZLIB);
 		return 0;
@@ -443,13 +558,17 @@ gbfeof(gbfile *file)
 	}
 }
 
+/*
+ * gbfungetc: (as fungetc)
+ */
+
 int
 gbfungetc(const int c, gbfile *file)
 {
 	int r = -1;
 	if (file->gzapi) {
 #if !ZLIB_INHIBITED
-		r = gzungetc(c, file->handle.gz);
+		file->back = -1;
 #else
 		fatal(NO_ZLIB);
 #endif
@@ -462,25 +581,33 @@ gbfungetc(const int c, gbfile *file)
 
 /* GPSBabel 'file' enhancements */
 
+/*
+ * gbfgetint32: read a signed 32-bit integer from input stream
+ */
+
 gbint32
 gbfgetint32(gbfile *file)
 {
 	char buf[4];
 	
-	gbfread(buf, 1, sizeof(buf), file);
-	
+	gbfread(&buf, 1, sizeof(buf), file);
+
 	if (file->big_endian)
 		return be_read32(buf);
 	else
 		return le_read32(buf);
 }
 
+/*
+ * gbfgetint16: read a signed 16-bit integer from input stream
+ */
+
 gbint16
 gbfgetint16(gbfile *file)
 {
 	char buf[2];
 	
-	gbfread(buf, 1, sizeof(buf), file);
+	gbfread(&buf, 1, sizeof(buf), file);
 	
 	if (file->big_endian)
 		return be_read16(buf);
@@ -488,54 +615,63 @@ gbfgetint16(gbfile *file)
 		return le_read16(buf);
 }
 
+/*
+ * gbfgetdbl: read a double value (8 byte, double precision) from input stream
+ */
+
 double 
 gbfgetdbl(gbfile *file)
 {
 	char buf[8];
-	gbfread(buf, 1, sizeof(buf), file);
-	return le_read_double(buf);
+
+	gbfread(&buf, 1, sizeof(buf), file);
+
+	return endian_read_double(buf, ! file->big_endian);
 }
 
+/*
+ * gbfgetflt: read a float value (4 byte, single precision) from input stream
+ */
+
 float
 gbfgetflt(gbfile *file)
 {
-	union {
-		float f;
-		gbint32 i;
-	} x;
+	char buf[4];
 	
-	x.i = gbfgetint32(file);
-	return x.f;
+	gbfread(&buf, 1, sizeof(buf), file);
+
+	return endian_read_float(buf, ! file->big_endian);
 }
 
 /*
  * gbfgetcstr: Reads a string from file until either a '\0' or eof.
  *             The result is a temporary allocated entity: use it or free it!
  */
+ 
 char *
 gbfgetcstr(gbfile *file)
 {
-	int len, size;
 	char *result;
+	int len = 0;
+	char *str = file->buff;
 	
-	len = size = 0;
-	result = xstrdup("");
-	
-	while (1) {
+	for (;;) {
 		char c = gbfgetc(file);
 		
 		if ((c == 0) || (c == EOF)) break;
 		
-		if (len == size) {
-			size += 32;
-			result = xrealloc(result, size + 1);
+		if (len == file->buffsz) {
+			file->buffsz += 64;
+			str = file->buff = xrealloc(file->buff, file->buffsz + 1);
 		}
-		result[len] = c;
+		str[len] = c;
 		len++;
 	}
 	
-	if ((len + 1) != size)
-		result = xrealloc(result, len + 1);
+	result = (char *) xmalloc(len + 1);
+	if (len > 0)
+		memcpy(result, str, len);
+	result[len] = '\0';
 	
 	return result;
 }
@@ -544,6 +680,7 @@ gbfgetcstr(gbfile *file)
  * gbfgetpstr: Reads a pascal string (first byte is length) from file.
  *             The result is a temporary allocated entity: use it or free it!
  */
+
 char *
 gbfgetpstr(gbfile *file)
 {
@@ -552,9 +689,9 @@ gbfgetpstr(gbfile *file)
 	
 	len = gbfgetc(file);
 	result = xmalloc(len + 1);
-	
-	if (len > 0)
+	if (len > 0) {
 		gbfread(result, 1, len, file);
+	}
 	result[len] = '\0';
 	
 	return result;
@@ -564,15 +701,14 @@ gbfgetpstr(gbfile *file)
  * gbfgetstr: Reads a string from file (util any type of line-breaks or eof or error)
  *            except xfree and free you can do all possible things with the result
  */
+
 char *
 gbfgetstr(gbfile *file)
 {
-	int len;
+	int len = 0;
 	char *result = file->line;
 	
-	len = file->linesz = 0;
-	
-	while (1) {
+	for (;;) {
 		char c = gbfgetc(file);
 		
 		if ((c == EOF) || (c == 0x1A)) {
@@ -583,24 +719,29 @@ gbfgetstr(gbfile *file)
 		}
 		else if (c == '\r') {
 			c = gbfgetc(file);
-			if ((c != '\n') && (c != EOF)) gbfungetc(c, file);
+			if ((c != '\n') && (c != EOF))
+				gbfungetc(c, file);
 			break;
 		}
 		else if (c == '\n') {
 			break;
 		}
 		if (len == file->linesz) {
-			file->linesz = len + 128;
-			result = file->line = xrealloc(file->line, len + 128 + 1);
+			file->linesz += 64;
+			result = file->line = xrealloc(file->line, file->linesz + 1);
 		}
 		result[len] = c;
 		len++;
 	}
-	result[len] = '\0'; // terminate resulting string
+	result[len] = '\0';	// terminate resulting string
 	
 	return result;
 }
 
+/*
+ * gbfputint16: write a signed 16-bit integer value into output stream
+ */
+ 
 int
 gbfputint16(const gbint16 i, gbfile *file)
 {
@@ -613,6 +754,10 @@ gbfputint16(const gbint16 i, gbfile *file)
 	return gbfwrite(buf, 1, sizeof(buf), file);
 }
 
+/*
+ * gbfputint32: write a signed 32-bit integer value into output stream
+ */
+
 int
 gbfputint32(const gbint32 i, gbfile *file)
 {
@@ -625,43 +770,68 @@ gbfputint32(const gbint32 i, gbfile *file)
 	return gbfwrite(buf, 1, sizeof(buf), file);
 }
 
+/*
+ * gbfputdbl: write a double value (8 byte, double precision) into output stream
+ */
+
 int
 gbfputdbl(const double d, gbfile *file)
 {
 	char buf[8];
-	le_write_double(buf, d );
+	
+	endian_write_double(buf, d, ! file->big_endian);
 	return gbfwrite(buf, 1, sizeof(buf), file);
 }
 
+/*
+ * gbfputflt: write a float value (4 byte, single precision) into output stream
+ */
+
 int 
 gbfputflt(const float f, gbfile *file)
 {
-	union {
-		float f;
-		gbint32 i; } x;
-		
-	x.f = f;
-	return gbfputint32(x.i, file);
+	char buf[4];
+	
+	endian_write_float(buf, f, ! file->big_endian);
+	return gbfwrite(buf, 1, sizeof(buf), file);
 }
 
+/*
+ * gbfputcstr: write a NULL terminated string into a stream (!) including NULL
+ *             return the number of written characters
+ */
+
 int 
 gbfputcstr(const char *s, gbfile *file)
 {
-	return gbfwrite(s, 1, strlen(s) + 1, file);
+	int len;
+	
+	len = (s == NULL) ? 0 : strlen(s);
+	if (len > 0) {
+		return gbfwrite(s, 1, len + 1, file);
+	} else {
+		gbfputc(0, file);
+		return 1;
+	}
 }
 
+/*
+ * gbfputcstr: write a pascal string into a stream
+ *             return the number of written characters
+ */
+
 int
 gbfputpstr(const char *s, gbfile *file)
 {
 	int len;
 	
-	len = strlen(s);
-	if (len > 255)
-		len = 255;
+	len = (s == NULL) ? 0 : strlen(s);
+	if (len > 255) len = 255;	/* the maximum size of a standard pascal string */
 	gbfputc(len, file);
-	gbfwrite(s, 1, len, file);
-	
-	return len + 1;
+	if (len > 0) {
+		gbfwrite(s, 1, len, file);
+	}
+	return (len + 1);
 }
 
-/* Thats all, sorry */
+/* Thats all, sorry. */
