@@ -20,6 +20,7 @@
  */
 
 #include "defs.h"
+#if PDBFMTS_ENABLED
 #include "coldsync/palm.h"
 #include "coldsync/pdb.h"
 #include "garmin_tables.h"
@@ -153,15 +154,15 @@ struct record
 static FILE *file_in;
 static FILE *file_out;
 static const char *out_fname;
-struct pdb *opdb;
-struct pdb_record *opdb_rec;
+static struct pdb *opdb;
+static struct pdb_record *opdb_rec;
 
 static char *dbname = NULL;
 
 static
 arglist_t my_args[] = {
-	{"dbname", &dbname, "Database name", NULL, ARGTYPE_STRING},
-	{0, 0, 0, 0, 0}
+	{"dbname", &dbname, "Database name", NULL, ARGTYPE_STRING, ARG_NOMINMAX},
+	ARG_TERMINATOR
 };
 
 static void
@@ -203,7 +204,7 @@ data_read(void)
 	struct record *rec;
 	struct pdb *pdb;
 	struct pdb_record *pdb_rec;
-	route_head *track_head;
+	route_head *track_head = NULL;
 
 	if (NULL == (pdb = pdb_Read(fileno(file_in)))) {
 		fatal(MYNAME ": pdb_Read failed\n");
@@ -232,7 +233,11 @@ data_read(void)
 		int lon;
 		int sz;
 		fi_t fi;
-                
+		int trk_num = 0;
+		int trk_seg_num = 1;
+		char trk_seg_num_buf[10];
+		char *trk_name = "";
+
 		wpt_tmp = waypt_new();
 
 		rec = (struct record *) pdb_rec->data;
@@ -273,8 +278,8 @@ data_read(void)
 			  wpt_tmp->depth = fi.f;
 			  fi.i = le_read32(&rec->wpt.d108.dist);
 			  wpt_tmp->proximity = fi.f;
-			  wpt_tmp->icon_descr_is_dynamic = 0;
-			  wpt_tmp->icon_descr = mps_find_desc_from_icon_number((rec->wpt.d108.smbl[1] << 8) + rec->wpt.d108.smbl[0], PCX);
+			  wpt_tmp->wpt_flags.icon_descr_is_dynamic = 0;
+			  wpt_tmp->icon_descr = gt_find_desc_from_icon_number((rec->wpt.d108.smbl[1] << 8) + rec->wpt.d108.smbl[0], PCX, NULL);
 			  waypt_add(wpt_tmp);
 			  break;
 
@@ -282,9 +287,7 @@ data_read(void)
 			 * CustomTrkHdr
 			 */
 			case 101:
-			  track_head = route_head_alloc();
-			  track_add_head(track_head);
-			  track_head->rte_name = xstrndup(rec->wpt.CustTrkHdr.name, sizeof(rec->wpt.CustTrkHdr.name));
+			  trk_name = rec->wpt.CustTrkHdr.name;
 			  sz = be_read16(&rec->wpt.CustTrkHdr.number);
               
 			  /* switch between custom track points and compact track points.
@@ -294,7 +297,27 @@ data_read(void)
 			  case 102:
 				tp_cust = (Custom_Trk_Point_Type *) ((char *) pdb_rec->data + sizeof(rec->header) + sizeof(rec->wpt.CustTrkHdr));
 				while (sz--) {
+				  if ((int)(tp_cust->new_trk) == 1 || trk_seg_num == 1) {
+					/* 
+					 * Start a new track segment
+					 */
+					track_head = route_head_alloc();
+					if (trk_seg_num == 1) {
+					  track_head->rte_name = xstrdup(trk_name);
+					} else {
+					  /* name in the form TRACKNAME #n */
+					  snprintf(trk_seg_num_buf, sizeof(trk_seg_num_buf), "%d", trk_seg_num);
+					  track_head->rte_name = xmalloc(strlen(trk_name)+strlen(trk_seg_num_buf)+3);
+					  sprintf(track_head->rte_name, "%s #%s", trk_name, trk_seg_num_buf);
+					}
+					trk_seg_num++;
+					track_head->rte_num = trk_num;
+					trk_num++;
+					track_add_head(track_head);
+				  }
+
 				  wpt_tmp = waypt_new();
+
 				  /* This is even more odd.
 				   * Track data is stored as big endian while
 				   * waypoint data is little endian!?
@@ -311,19 +334,38 @@ data_read(void)
 				  wpt_tmp->creation_time = be_read32(&tp_cust->time) + 631065600;
 				  fi.i = be_read32(&tp_cust->alt);
 				  wpt_tmp->altitude = fi.f;
-				  route_add_wpt(track_head, wpt_tmp);
+				  track_add_wpt(track_head, wpt_tmp);
 				  tp_cust++;
 				}
 				break;
 			  case 104:
 				tp_comp = (Compact_Trk_Point_Type *) ((char *) pdb_rec->data + sizeof(rec->header) + sizeof(rec->wpt.CustTrkHdr));
 				while (sz--) {
+				  if ((int)(tp_comp->new_trk) == 1 || trk_seg_num == 1) {
+					/* 
+					 * Start a new track segment
+					 */
+					track_head = route_head_alloc();
+					if (trk_seg_num == 1) {
+					  track_head->rte_name = xstrdup(trk_name);
+					} else {
+					  /* name in the form TRACKNAME #n */
+					  snprintf(trk_seg_num_buf, sizeof(trk_seg_num_buf), "%d", trk_seg_num);
+					  track_head->rte_name = xmalloc(strlen(trk_name)+strlen(trk_seg_num_buf)+3);
+					  sprintf(track_head->rte_name, "%s #%s", trk_name, trk_seg_num_buf);
+					}
+					trk_seg_num++;
+					track_head->rte_num = trk_num;
+					trk_num++;
+					track_add_head(track_head);
+				  }
+
 				  wpt_tmp = waypt_new();
 				  lon = be_read32(&tp_comp->lon);
 				  lat = be_read32(&tp_comp->lat);
 				  wpt_tmp->longitude = lon / 2147483648.0 * 180.0;
 				  wpt_tmp->latitude = lat / 2147483648.0 * 180.0;
-				  route_add_wpt(track_head, wpt_tmp);
+				  track_add_wpt(track_head, wpt_tmp);
 				  tp_comp++;
 				}
 				break;
@@ -364,7 +406,7 @@ my_write_wpt(const waypoint *wpt)
 	le_write32(&rec->wpt.d103.lat, lat);
 	le_write32(&rec->wpt.d103.lon, lon);
 
-	opdb_rec = new_Record(0, 0, ct++, vdata - (char *) rec, (const ubyte *) rec);
+	opdb_rec = new_Record(0, 0, ct++, (uword) (vdata - (char *) rec), (const ubyte *) rec);
 
 	if (opdb_rec == NULL) {
 		fatal(MYNAME ": libpdb couldn't create record\n");
@@ -408,6 +450,7 @@ data_write(void)
 
 ff_vecs_t gpilots_vecs = {
 	ff_type_file,
+	FF_CAP_RW_WPT,
 	rd_init,
 	wr_init,
 	rd_deinit,
@@ -415,5 +458,7 @@ ff_vecs_t gpilots_vecs = {
 	data_read,
 	data_write,
 	NULL, 
-	my_args
+	my_args,
+	CET_CHARSET_ASCII, 0	/* CET-REVIEW */
 };
+#endif
