@@ -25,13 +25,14 @@ void   GPS_Util_Put_Uint(UC *s, const uint32 v);
 uint32 GPS_Util_Get_Uint(const UC *s);
 
 void   GPS_Warning(char *s);
-void   GPS_Error(char *s);
+void   GPS_Error(char *fmt, ...);
+void   GPS_Serial_Error(const char *hdr, ...);
 void   GPS_Fatal(char *s);
 void   GPS_Enable_Error(void);
 void   GPS_Enable_Warning(void);
 void   GPS_Disable_Error(void);
 void   GPS_Disable_Warning(void);
-void   GPS_User(char *s);
+void   GPS_User(const char *fmt, ...);
 void   GPS_Disable_User(void);
 void   GPS_Enable_User(void);
 void   GPS_Diagnose(int32 c);
