@@ -26,6 +26,7 @@
 #include <ctype.h>
 #include "defs.h"
 #include "csv_util.h"
+#include "jeeps/gpsmath.h"
 
 #if CSVFMTS_ENABLED
 #define MYNAME	"XCSV"
@@ -38,6 +39,7 @@ static char *snupperopt = NULL;
 static char *snuniqueopt = NULL;
 char *prefer_shortnames = NULL;
 char *xcsv_urlbase = NULL;
+static char *opt_datum;
 
 static const char *intstylebuf = NULL;
 
@@ -58,6 +60,8 @@ arglist_t xcsv_args[] = {
 	{"prefer_shortnames", &prefer_shortnames, 
 		"Use shortname instead of description", 
 		NULL, ARGTYPE_BOOL, ARG_NOMINMAX },
+	{"datum", &opt_datum, "GPS datum (def. WGS 84)", 
+		NULL, ARGTYPE_STRING, ARG_NOMINMAX}, 
 	ARG_TERMINATOR
 };
 
@@ -327,6 +331,31 @@ xcsv_parse_style_line(const char *sbuff)
 	    cet_convert_init(p, 1);
 	    xfree(p);
 	} else
+
+	if (ISSTOKEN(sbuff, "DATUM")) {
+	    p = csv_stringtrim(&sbuff[8], "\"", 1);
+	    xcsv_file.gps_datum = GPS_Lookup_Datum_Index(p);
+	    is_fatal(xcsv_file.gps_datum < 0, MYNAME ": datum \"%s\" is not supported.", p);
+	    xfree(p);
+	} else
+
+	if (ISSTOKEN(sbuff, "DATATYPE")) {
+	    p = csv_stringtrim(&sbuff[8], "\"", 1);
+	    if (case_ignore_strcmp(p, "TRACK") == 0) {
+		xcsv_file.datatype = trkdata;
+	    }
+	    else if (case_ignore_strcmp(p, "ROUTE") == 0) {
+		xcsv_file.datatype = rtedata;
+	    }
+	    else if (case_ignore_strcmp(p, "WAYPOINT") == 0) {
+		xcsv_file.datatype = wptdata;
+	    }
+	    else {
+		fatal(MYNAME ": Unknown data type \"%s\"!\n", p);
+	    }
+	    xfree(p);
+	
+	} else
 	
 	if (ISSTOKEN(sbuff, "IFIELD")) {
 	    key = val = pfc = NULL;
@@ -510,11 +539,17 @@ xcsv_rd_init(const char *fname)
         xcsv_read_style(styleopt);
     }
 
-    if (global_opts.masked_objective & (TRKDATAMASK|RTEDATAMASK)) {
-	warning(MYNAME " attempt to read %s as a track or route, but this module only supports waypoints on read.  Reading as waypoints instead.\n", fname);
+    if ((xcsv_file.datatype == 0) || (xcsv_file.datatype == wptdata)) {
+	if (global_opts.masked_objective & (TRKDATAMASK|RTEDATAMASK)) {
+	    warning(MYNAME " attempt to read %s as a track or route, but this format only supports waypoints on read.  Reading as waypoints instead.\n", fname);
+	}
     }
 
     xcsv_file.xcsvfp = gbfopen(fname, "r", MYNAME);
+    if (opt_datum) {
+	xcsv_file.gps_datum = GPS_Lookup_Datum_Index(opt_datum);
+	is_fatal(xcsv_file.gps_datum < 0, MYNAME ": datum \"%s\" is not supported.", opt_datum);
+    }
 
 }
 
@@ -564,7 +599,10 @@ xcsv_wr_init(const char *fname)
         setshort_badchars(xcsv_file.mkshort_handle, xcsv_file.badchars);
 
     }
-
+    if (opt_datum) {
+	xcsv_file.gps_datum = GPS_Lookup_Datum_Index(opt_datum);
+	is_fatal(xcsv_file.gps_datum < 0, MYNAME ": datum \"%s\" is not supported.", opt_datum);
+    }
 }
 
 static void
